package se.su.dsv.scipro.reviewing;

import org.joda.time.Instant;
import scala.Option;
import scala.runtime.AbstractFunction0;
import se.su.dsv.scipro.file.FileDescription;
import se.su.dsv.scipro.misc.DaysService;

import javax.persistence.*;
import java.util.Date;

@Entity
@SuppressWarnings("PMD.UnusedPrivateField")
public class Decision {
    @Id
    @GeneratedValue
    private Long id;

    @Enumerated(EnumType.STRING)
    private Status status = Status.UNDECIDED;

    @OneToOne(optional = false)
    private FileDescription thesis;

    @Basic
    private String reason;

    @Basic
    private String comment;

    @OneToOne(optional = true)
    private FileDescription attachment;

    @OneToOne(optional = true)
    private FileDescription gradingReport;

    @Temporal(TemporalType.TIMESTAMP)
    private Date requested;

    @Temporal(TemporalType.TIMESTAMP)
    private Date decisionDate;

    @ManyToOne(optional = false)
    private ReviewerApproval reviewerApproval;

    protected Decision() {} // JPA

    Decision(ReviewerApproval reviewerApproval, final FileDescription thesis, final String comment) {
        this(reviewerApproval, thesis, Instant.now(), comment);
    }

    public Decision(ReviewerApproval reviewerApproval, final FileDescription thesis, final Instant requested, final String comment) {
        this.reviewerApproval = reviewerApproval;
        this.thesis = thesis;
        this.requested = requested.toDate();
        this.comment = comment;
    }

    public FileDescription getThesis() {
        return thesis;
    }

    public ReviewerApproval getReviewerApproval() {
        return reviewerApproval;
    }

    public Status getStatus() {
        return status;
    }

    public String getReason() {
        return reason;
    }

    public String getComment() {
        return comment;
    }

    public Option<FileDescription> getAttachment() {
        return Option.apply(attachment);
    }

    public Option<FileDescription> getGradingReport() {
        return Option.apply(gradingReport);
    }

    public Date getRequested() {
        return requested;
    }

    public Date getDecisionDate() {
        return decisionDate;
    }

    void approve(final String reason, final Option<FileDescription> attachment, final Option<FileDescription> gradingReport) {
        decide(Status.APPROVED, reason, attachment, gradingReport);
    }

    void reject(final String reason, final Option<FileDescription> attachment, final Option<FileDescription> gradingReport) {
        decide(Status.REJECTED, reason, attachment, gradingReport);
    }

    boolean isLate(final DaysService daysService, final int daysToDecide) {
        return decisionDate().isAfter(getDeadline(daysService, daysToDecide));
    }

    public Instant getDeadline(final DaysService daysService, final int daysToDecide) {
        return new Instant(daysService.workDaysAfter(this.requested,daysToDecide));
    }

    private Instant decisionDate() {
        return decisionDate == null ? Instant.now() : new Instant(decisionDate);
    }

    private void decide(Status status, final String reason, final Option<FileDescription> attachment, final Option<FileDescription> gradingReport) {
        decideWithDecisionDate(status, reason, attachment, gradingReport, Instant.now());
    }

    void decideWithDecisionDate(final Status status, final String reason, final Option<FileDescription> attachment, final Option<FileDescription> gradingReport, final Instant decisionDate) {
        if (this.status != Status.UNDECIDED) {
            throw new IllegalStateException();
        }
        this.decisionDate = decisionDate.toDate();
        this.status = status;
        this.reason = reason;
        this.attachment = attachment.getOrElse(new AbstractFunction0<FileDescription>() {
            @Override
            public FileDescription apply() {
                return null;
            }
        });
        this.gradingReport = gradingReport.getOrElse(new AbstractFunction0<FileDescription>() {
            @Override
            public FileDescription apply() {
                return null;
            }
        });
    }
}
